"""Widget modules for sqv."""

from sqv.widgets.data_viewer import DataViewerTab
from sqv.widgets.sql_editor import SQLTab
from sqv.widgets.structure import StructureTab

__all__ = ["StructureTab", "DataViewerTab", "SQLTab"]
